/* please set your TABs to 4 to get proper formatting */

/*
           panteltje ptlrc (short for PanTeLtje RS232 Communication)

         Copyright (C) Jan Panteltje 2008-always

 email: panteltje@yahoo.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "ptlrc.h"


char *strsave(char *s) /* save char array s somewhere */
{
char *p;

p = malloc( strlen(s) +  1);
if(! p) return 0;

strcpy(p, s);
return p;
} /* end function strsave */




int set_port_parameters(int fd, int baudrate, char *device)
{
struct termios tty;
speed_t baud;

if(debug_flag)
	{
	fprintf(stdout, "set_port_parameters(): arg fd=%d baudrate=%d device=%s\n", fd, baudrate, device);
	}

/* convert real baudrates to 'B' baudrates */
switch(baudrate)
	{
	case 75:
		baud = B75;
		break;
	case 150:
		baud = B150;
		break;
	case 300:
		baud = B300;
		break;
	case 600:
		baud = B600;
		break;	
	case 1200:
		baud = B1200;
		break;
	case 2400:
		baud = B2400;
		break;
	case 4800:
		baud = B4800;
		break;	
	case 9600:
		baud = B9600;
		break;
	case 19200:
		baud = B19200;
		break;
	case 38400:
		baud = B38400;
		break;
	case 57600:
		baud = B57600;
		break;
	case 115200:
		baud = B115200;
		break;
	default:
		fprintf(stderr, "ptlrc: baudrate=%d unsupported, aborting.\n", baudrate);
		return 0;
		break;
	} /* end switch baudrate */

/* set comport parameters */
tcgetattr(fd, &tty);

tty.c_cflag = 0;
tty.c_cflag |= CLOCAL;			/* ignore modem control lines */
tty.c_cflag |= CREAD;			/* enable receiver */
tty.c_cflag |= CS8;				/* use 8 data bits */
//tty.c_cflag |= CSTOPB;		/* 2 stop bits */
//tty.c_cflag |= PARENB;		/* enable parity */
//tty.c_cflag |= PARODD;		/* use odd parity */
//tty.c_cflag &= ~CRTSCTS;		/* do not use RTS and CTS handshake */

tty.c_iflag = 0;
tty.c_iflag |= IGNBRK;
tty.c_iflag |= IGNPAR;
//tty.c_iflag |= PARMRK
//tty.c_iflag |= INPCK;
//tty.c_iflag |= INLCR;
//tty.c_iflag |= ISTRIP
//tty.c_iflag &= ~(IXON | IXOFF | IXANY);

tty.c_oflag = 0;

tty.c_lflag = 0;
//tty.c_lflag |= ICANON;
//tty.c_cc[VMIN] = 1;
//tty.c_cc[VTIME] = 5;
//tty.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);


cfsetospeed(&tty, (speed_t)baud);
cfsetispeed(&tty, (speed_t)baud);

tcsetattr(fd, TCSANOW, &tty);

return 1;
} /* end function set_port_parameters */


void print_usage()
{
fprintf(stdout, "Panteltje ptlrc-%s serial communication program.\n", VERSION);
fprintf(stdout, "Usage: ptlrc [-b baudrate] [-d device_name] [-h] [-i] [-v]\n");
fprintf(stdout,\
"\
b n       baudrate, either 75, 150, 300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, or 115200, default 19200.\n\
d device  device name, default /dev/ttyS1.\n\
i         script flag, use this to send single command from a script,\n\
           example: echo a1 | ptlrc -i , or echo a1 | ptlrc -i > result.\n\
h         help (this help).\n\
v         verbose, prints functions and arguments.\n\
Escape exits.\n\
Example:\n\
ptlrc -d /dev/ttyUSB1 -b 9600 | tee temp.log\n\
");

} /* end function print_usage */



int main(int argc, char **argv)
{
int a, b, i;
int fd;
fd_set readfs; /* data type represents file descriptor for select function, (actually it is a bit array) */
struct timeval timeout;
char rx_buffer[1024];
char tx_buffer[1024];
int baudrate;
int script_flag;
device = strsave("/dev/ttyS1");
if(! device)
	{
	fprintf(stderr, "ptlrc: main(): could not allocate space for device name, aborting.\n");
				
	exit(1);
	}


/* select con io 1 byte at the time, so we can use tee */
setbuf(stdout, NULL);
//setbuf(stdin, NULL);

/* defaults */


/* set com parameters 19200 8 bit no parity */
baudrate = 19200;
script_flag = 0;
/* end defaults */

while(1)
	{
	a = getopt(argc, argv, "b:d:ihv");
	if(a == -1) break;

	switch(a)
		{
		case 'b':
			baudrate = atoi(optarg);
			break;			
		case 'h': // help
			print_usage();
			exit(1);
			break;
		case 'i':
			script_flag = 1;
			break;
		case 'd':
			free(device);
			device = strsave(optarg);
			if(! device)
				{
				fprintf(stderr, "ptlrc: main(): could not allocate space for device name, aborting.\n");
				
				exit(1);
				}
			break;
		case 'v': // debug
			debug_flag = 1;
			break;
        case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "ptlrc: unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "ptlrc: nknown option character `\\x%x'.\n",\
				optopt);
				}
			print_usage();

			exit(1);
			break;			
		default:
			print_usage();

			exit(1);
			break;
		} /* end switch a */
	} /* end while getopt() */

fd = open(device, O_RDWR);
if(fd  == -1)
	{
	fprintf(stderr, "ptlrc: could not open device %s for read-write, aborting.\n", device);
	
	exit(1);
	}

// int set_port_parameters(int fd, int baudrate, char *device)
if(! set_port_parameters(fd, baudrate, device) )
	{
	fprintf(stderr, "ptlrc: could not set baudrate %d, aborting.\n", baudrate); 
	
	exit(1);
	}

if(! script_flag) 
	{
	/* ID */
	fprintf(stdout, "Panteltje ptlrc-%s using device %s\n", VERSION, device);

	/* How to abort */
	fprintf(stdout, "Escape exits.\n");

	/* no wait for ENTER */
	system("stty raw");

	/* no echo */
	system("stty -echo");
	} /* end if ! script_flag */

/* communications loop */
while(1)
	{
	/* this macro initializes the file descriptor readfs to to be the empty set */
	FD_ZERO(&readfs);

	/* this macro adds fd to the file descriptor readfs */
	FD_SET(fd, &readfs);

	/* this macro adds fileno(stdin) to the file descriptor readfs */
	FD_SET(fileno(stdin), &readfs);

	timeout.tv_sec = 100;
	timeout.tv_usec = 0;

	/* int FD_SETSIZE  macro is maximum number of filedescriptors that fd_set can hold */
 	/* function select waits for specified filedescr. to have a signal */
	/* last argument struct timeval *timeout */
	b = select(FD_SETSIZE, &readfs, NULL, NULL, &timeout);
	if(b == -1) /* 0 is timeout, -1 error (in errno) */
		{
		fprintf(stderr, "ptlrc: select: failed, because ");
		perror("");
		fprintf(stderr, "aborting.\n");

		exit(1);		
		}

	/* test if user has data */
	if( FD_ISSET(fileno(stdin), &readfs) )
		{	
		a = read(fileno(stdin), tx_buffer, 1023);
		for(i = 0; i < a; i++)
			{
			if(tx_buffer[i] == 27) /* Escape */
				{
				if(! script_flag) system("stty sane");

				exit(1);
				}

			/* PIC wants CR */
			if(tx_buffer[i] == 10) tx_buffer[i] = 13;

			} /* end for all available characters */

//		ssize_t write(int fd, const void *buf, size_t count);
		b = write(fd, tx_buffer, a);
		if(b == -1)
			{
			fprintf(stderr, "ptlrc: write to device %s failed because ", device);
			perror("");				

			fprintf(stderr, "aborting.\n");
			
			if(! script_flag) system("stty sane");

			exit(1);
			}

		if(b != a)
			{
			fprintf(stderr, "ptlrc: could only write %d of %d bytes to device %s.\n", b, a, device);
			}

		} /* end if user has data */

	/* test if remote side has data */
	if( FD_ISSET(fd, &readfs) )
		{
//		ssize_t read(int fd, void *buf, size_t count);
		a = read(fd, rx_buffer, 1023);

		for(i = 0; i < a; i++)
			{
			fprintf(stdout, "%c", rx_buffer[i]);		

			if(script_flag)
				{
				if(rx_buffer[i] == 10) exit(0);
				}
			}
		} /* end for all available characters from remote side */
	
	} /* end while com loop */

exit(0);
} /* end main */



